#!/usr/bin/python

import eon_read
import optparse
import os
from optparse import OptionParser
import subprocess
import shutil
import write_statelist
import numpy
import math

parser = OptionParser()
parser.add_option("-d", "--dir",type="string", dest="in_dir", help="Path of the directory eon results", metavar="DIR",default=os.getcwd())
parser.add_option("-T","--temperature", type="float", dest="temperature",help="temperature")
(options, args) = parser.parse_args()



if not options.in_dir:
    print "ERROR: specify input directory"
    exit(0)

in_path=    os.path.join(os.getcwd(),options.in_dir)

if not os.path.isdir(in_path):
    print "ERROR: '%s' is not a valid directory" % in_path
    exit(0)
if not os.path.isdir(os.path.join(in_path,"states")):
    print "ERROR: '%s' is not a valid EON directory" % in_path
    exit(0)
if not options.temperature:
    print "ERROR: temperature not set" 
    exit(0)

#######################################################################################################

def calc_prefactor(modes_reactant, modes_saddle):
    assert(len(modes_reactant)==len(modes_saddle) )

    prefactor = 1.0
    for i in range(len(modes_saddle)):
        prefactor *= modes_reactant[i]
        if(modes_saddle[i] > 0.0):
            prefactor /= modes_saddle[i]
    
    prefactor = math.sqrt(prefactor) / (2*math.pi * 10.18e-15)
    return prefactor

#######################################################################################################

def read_modes(filename):
    modes = []
    modefile = open(filename)
    for frequency in modefile:
        modes.append(float(frequency))
    modefile.close()
    
    return modes

#######################################################################################################


def find_reverse_proc(statelist,stateid,procid):
    reverse_proc_id=None
    eps = 0.000001
    proc = statelist[ stateid ]['proclist'][procid]
    product_id = proc['productid']

    #if the process is to itsself:
    if proc['reactantid'] == proc['productid']:
        for candidate in statelist[ product_id ]['proclist']:
            if 'calculated_prefactor' in candidate:
                continue
            if ( abs(1.0 - candidate['saddle_energy']/proc['saddle_energy'] ) < eps ):
                reverse_proc_id = candidate['procid']
                break
    else:
        for candidate in statelist[ product_id ]['proclist']:
            if candidate['productid'] != stateid:
                continue
            #if the prefactor was already set for this process, it cannot count as the reverse
            if 'calculated_prefactor' in candidate:
                continue


            if(  ( abs(1.0 - candidate['prefactor']/proc['product_prefactor'] ) < eps ) and
                 ( abs(1.0 - candidate['product_prefactor']/proc['prefactor'] ) < eps ) and
                 ( abs(1.0 - candidate['saddle_energy']/proc['saddle_energy'] ) < eps )  ):

                reverse_proc_id = candidate['procid']
                break
    
    return reverse_proc_id

#######################################################################################################

def main():
    statelist = eon_read.read_statelist(in_path,read_processes=True)
    kT= options.temperature/11604.50529
    #read default mode for saddle.
    defaultmodesfilename = os.path.join(in_path,"modes_saddle_default.dat")
    defaultmode = read_modes(defaultmodesfilename)

    #read exact modefiles.
    for stateid in range(len(statelist)):
        modefilename = os.path.join(in_path,"states",str(stateid),"mode_exact.dat")
        statelist[stateid]['modes']= read_modes(modefilename)
        print "Number of modes in state %d: %d" % (stateid,len(statelist[stateid]['modes'])) 
        assert(len(statelist[stateid]['modes']) == int(3*statelist[stateid]['reactant_config'].free.sum()) )
        

    #read all process modes and calculate rates.
    for stateid in range(len(statelist)):
        for procid in range(statelist[stateid]['numprocs']):
        
            proc = statelist[stateid]['proclist'][procid]
            productid = proc['productid']
            if 'calculated_rate' in proc:
                continue

            if productid == -1:
                #make sure the rate is correct (in case we are changing the temperature.
                proc['rate'] = proc['prefactor']*math.exp(-proc['barrier'] / kT )
                continue
                        
            #read saddle modes
            modefilename = os.path.join(in_path,"states",str(stateid),"procdata","mode_exact_"+str(proc['procid'])+".dat")
            proc['saddlemodes'] = read_modes(modefilename)
            if len(proc['saddlemodes']) != int(3*statelist[stateid]['reactant_config'].free.sum()):
                print "\tERROR: no valid modefile for state %d proc %d" % (stateid,proc['procid'])
                print "\tSetting default mode"
                proc['saddlemodes'] = defaultmode[:]           
       
            reverse_proc_id = find_reverse_proc(statelist,stateid,procid)
            
            
            if reverse_proc_id==None:
                print "Could not find reverse process for proc %d from state %d" % (procid,stateid)
                exit(0)

            reverse_proc = statelist[productid]['proclist'][reverse_proc_id]
            if 'calculated_rate' in reverse_proc:
                print "ERROR: already calculated rate for proc %d from state %d, reverse proc identification not unique?" % (reverse_proc_id,productid)
            
            saddle_energy = proc['saddle_energy']        
            reactant_energy = statelist[stateid]['energy']
            product_energy = statelist[ productid ]['energy']

            #calculate prefactors and rates
            forward_prefactor   = calc_prefactor(statelist[stateid]['modes'],proc['saddlemodes'])
            forward_rate        = forward_prefactor*math.exp(-(saddle_energy-reactant_energy) / kT )
            backward_prefactor  = calc_prefactor(statelist[productid]['modes'],proc['saddlemodes'])
            backward_rate       = backward_prefactor*math.exp(-(saddle_energy-product_energy) / kT )


            proc['prefactor'] = forward_prefactor
            proc['product_prefactor'] = backward_prefactor
            proc['rate'] = forward_rate
            proc['barrier'] = saddle_energy - reactant_energy
            
            reverse_proc['prefactor'] = backward_prefactor
            reverse_proc['product_prefactor'] = forward_prefactor
            reverse_proc['rate'] = backward_rate
            reverse_proc['barrier'] = saddle_energy - product_energy
            
            proc['calculated_rate'] = True
            reverse_proc['calculated_rate'] = True

            
    print " " 
    print "writing processtables..."          
    for state in statelist:
        statepath = os.path.join(in_path,"states", str(state['stateid']) )
        write_statelist.write_processtable(state,statepath,overwrite=True)


        


#######################################################################################################

if __name__ == "__main__":
    main()

